#include "glwidget.h"

#include <QDebug>
#include <QVector3D>
#include <QVector>
#include <QOpenGLBuffer>
#include <QOpenGLFunctions>

GLWidget::GLWidget(QWidget* parent, QList<Kostka*>& kostky) : QGLWidget(parent)
  , viewCenterFixed(false)
  , m_panAngle(0.0f)
  , m_tiltAngle(0.0f)
  , panAngle(0.0f)
  , tiltAngle(0.0f)
  , camera_zoom(25.0f)
  , aspectRatio(float(width()) / float(height()))
  , nearPlane(0.01f)
  , farPlane(256.0f)
  , time(0.0f)
  , leftButtonPressed(false)
  , rightButtonPressed(false)
  , mouse_position(QPoint(0, 0))
{
    xRot = 0;
    yRot = 0;
    zRot = 0;

    setFocusPolicy(Qt::StrongFocus);
    setMouseTracking(true);

    kosticky = kostky;

    floorSel = 0;
    floorSel2 = 0;

    startTimer(16);
}

GLWidget::~GLWidget()
{
}

void GLWidget::rotateBy(int xAngle, int yAngle, int zAngle)
{
    xRot += xAngle;
    yRot += yAngle;
    zRot += zAngle;
    updateGL();
}

void GLWidget::initializeGL()
{
    this->makeCurrent();

    glShadeModel(GL_FLAT);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_CULL_FACE);

    glClearColor(0.65f, 0.77f, 1.0f, 1.0f);
}

void GLWidget::paintGL()
{
    this->makeCurrent();

    qglClearColor(Qt::black);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glTranslatef(0.0f, 0.0f, -100.0f);
    glRotatef(xRot / 16.0f, 1.0f, 0.0f, 0.0f);
    glRotatef(yRot / 16.0f, 0.0f, 1.0f, 0.0f);
    glRotatef(zRot / 16.0f, 0.0f, 0.0f, 1.0f);

    foreach(Kostka* kostka, kosticky)
    {
        float floor = (float)kostka->Floor;
        float X     = (float)kostka->X;
        float Y     = (float)kostka->Y;

        QString bonusX = kostka->Size[0];
        QString bonusY = kostka->Size[2];

        float X2 = bonusX.toFloat();
        float Y2 = bonusY.toFloat();

        float red = 0.0f;
        float green = 0.0f;
        float blue = 0.0f;

        if(kostka->Color == 'A')
        {
            red = 0.0f;
            green = 1.0f;
            blue = 1.0f;
        }
        else if(kostka->Color == 'R')
        {
            red = 1.0f;
            green = 0.0f;
            blue = 0.0f;
        }
        else if(kostka->Color == 'G')
        {
            red = 0.0f;
            green = 1.0f;
            blue = 0.0f;
        }
        else if(kostka->Color == 'B')
        {
            red = 0.0f;
            green = 0.0f;
            blue = 1.0f;
        }
        else if(kostka->Color == 'Y')
        {
            red = 1.0f;
            green = 1.0f;
            blue = 0.0f;
        }

        if(kostka->Floor == floorSel)
        {
            red = 1.0f;
            green = 1.0f;
            blue = 1.0f;
        }

        if(kostka->Floor != floorSel2 && floorSel2 != 0)
            continue;

        glBegin(GL_QUADS);                // Begin drawing the color cube with 6 quads
              // Top face (y = 1.0f)
              // Define vertices in counter-clockwise (CCW) order with normal pointing out
              glColor3f(red, green, blue);     // Green
              glVertex3f(X+X2, floor, Y);
              glVertex3f(X, floor, Y);
              glVertex3f(X, floor, Y+Y2);
              glVertex3f(X+X2, floor, Y+Y2);

              // Bottom face (y = -1.0f)
              glColor3f(red, green, blue);      // Orange
              glVertex3f(X+X2, floor-1.0f, Y+Y2);
              glVertex3f(X, floor-1.0f, Y+Y2);
              glVertex3f(X, floor-1.0f, Y);
              glVertex3f(X+X2, floor-1.0f, Y);

              // Front face  (z = 1.0f)
              glColor3f(red, green, blue);      // Red
              glVertex3f(X+X2, floor, Y+Y2);
              glVertex3f(X, floor, Y+Y2);
              glVertex3f(X, floor-1.0f, Y+Y2);
              glVertex3f(X+X2, floor-1.0f, Y+Y2);

              // Back face (z = -1.0f)
              glColor3f(red, green, blue);      // Yellow
              glVertex3f(X+X2, floor-1.0f, Y);
              glVertex3f(X, floor-1.0f, Y);
              glVertex3f(X, floor, Y);
              glVertex3f(X+X2, floor, Y);

              // Left face (x = -1.0f)
              glColor3f(red, green, blue);      // Blue
              glVertex3f(X, floor, Y+Y2);
              glVertex3f(X, floor, Y);
              glVertex3f(X, floor-1.0f, Y);
              glVertex3f(X, floor-1.0f, Y+Y2);

              // Right face (x = 1.0f)
              glColor3f(red, green, blue);     // Magenta
              glVertex3f(X+X2, floor, Y);
              glVertex3f(X+X2, floor, Y+Y2);
              glVertex3f(X+X2, floor-1.0f, Y+Y2);
              glVertex3f(X+X2, floor-1.0f, Y);
           glEnd();  // End of drawing color-cube

        glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);

        glBegin(GL_QUADS);                // Begin drawing the color cube with 6 quads
              // Top face (y = 1.0f)
              // Define vertices in counter-clockwise (CCW) order with normal pointing out
              glColor3f(0.0f, 0.0f, 0.0f);     // Green
              glVertex3f(X+X2, floor, Y);
              glVertex3f(X, floor, Y);
              glVertex3f(X, floor, Y+Y2);
              glVertex3f(X+X2, floor, Y+Y2);

              // Bottom face (y = -1.0f)
              glColor3f(0.0f, 0.0f, 0.0f);
              glVertex3f(X+X2, floor-1.0f, Y+Y2);
              glVertex3f(X, floor-1.0f, Y+Y2);
              glVertex3f(X, floor-1.0f, Y);
              glVertex3f(X+X2, floor-1.0f, Y);

              // Front face  (z = 1.0f)
              glColor3f(0.0f, 0.0f, 0.0f);
              glVertex3f(X+X2, floor, Y+Y2);
              glVertex3f(X, floor, Y+Y2);
              glVertex3f(X, floor-1.0f, Y+Y2);
              glVertex3f(X+X2, floor-1.0f, Y+Y2);

              // Back face (z = -1.0f)
              glColor3f(0.0f, 0.0f, 0.0f);
              glVertex3f(X+X2, floor-1.0f, Y);
              glVertex3f(X, floor-1.0f, Y);
              glVertex3f(X, floor, Y);
              glVertex3f(X+X2, floor, Y);

              // Left face (x = -1.0f)
              glColor3f(0.0f, 0.0f, 0.0f);
              glVertex3f(X, floor, Y+Y2);
              glVertex3f(X, floor, Y);
              glVertex3f(X, floor-1.0f, Y);
              glVertex3f(X, floor-1.0f, Y+Y2);

              // Right face (x = 1.0f)
              glColor3f(0.0f, 0.0f, 0.0f);
              glVertex3f(X+X2, floor, Y);
              glVertex3f(X+X2, floor, Y+Y2);
              glVertex3f(X+X2, floor-1.0f, Y+Y2);
              glVertex3f(X+X2, floor-1.0f, Y);
           glEnd();  // End of drawing color-cube

           glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
    }
}

void GLWidget::resizeGL(int width, int height)
{
    this->makeCurrent();

    GLfloat aspect = (GLfloat)width / (GLfloat)height;

       // Set the viewport to cover the new window
       glViewport(0, 0, width, height);

       // Set the aspect ratio of the clipping volume to match the viewport
       glMatrixMode(GL_PROJECTION);  // To operate on the Projection matrix
       glLoadIdentity();             // Reset

       // Set up orthographic projection view [NEW]
       if (width >= height) {
         // aspect >= 1, set the height from -1 to 1, with larger width
          glOrtho(-3.0 * aspect, 3.0 * aspect, -3.0, 3.0, 0.1, 100);
       } else {
          // aspect < 1, set the width to -1 to 1, with larger height
         glOrtho(-3.0, 3.0, -3.0 / aspect, 3.0 / aspect, 0.1, 100);
       }
}

void GLWidget::update(float t)
{

}

void GLWidget::mousePressEvent(QMouseEvent* e)
{
    lastPos = e->pos();
}

void GLWidget::mouseReleaseEvent(QMouseEvent* e)
{
}

void GLWidget::mouseMoveEvent(QMouseEvent* e)
{
    int dx = e->x() - lastPos.x();
    int dy = e->y() - lastPos.y();

    if (e->buttons() & Qt::LeftButton) {
        rotateBy(8 * dy, 8 * dx, 0);
    } else if (e->buttons() & Qt::RightButton) {
        rotateBy(8 * dy, 0, 8 * dx);
    }
    lastPos = e->pos();
}

void GLWidget::timerEvent(QTimerEvent* e)
{
    this->makeCurrent();

    update(0.0f);
    updateGL();
}

void GLWidget::selectFloor(int fl)
{
    floorSel = fl;
}

void GLWidget::selectFloor2(int fl)
{
    floorSel2 = fl;
}
